# plot-all-figures.jl
#
# Generates all the plots, figures, etc., for the phase retrieval paper, saving
# them as needed.


using PyPlot;
if (!isdefined(:IP))
  include("image-processing.jl");
end
if (!isdefined(:PhaseExperimentPlotting))
  include("phase-experiment-plotting.jl");
end
loadpath = "./CompositeResults/";

PEP = PhaseExperimentPlotting;

# ############################################################
# ######### ---- Part 1: Zero-noise experiments ---- #########
# ############################################################

if (run_zero_noise)
  files_400 =
    [ "pz_dim400_tests400_nvals10_seed4634_prox_lowtol_signs_biginit.txt",
      "pz_dim400_tests400_nvals10_seed4634_taf_lowtol_signs_biginit.txt" ];

  files_800 =
    [ "pz_dim800_tests400_nvals10_seed4634_prox_lowtol_signs_biginit.txt",
      "pz_dim800_tests400_nvals10_seed4634_taf_lowtol_signs_biginit.txt" ];

  PEP.CompareZeroNoiseExperiments(files_800[1],
                                  files_800[2],
                                  loadpath = loadpath,
                                  num_n_vals = 10,
                                  dim = 800, num_tests_per = 400,
                                  plot_name_a = "Prox",
                                  plot_name_b = "TAF",
                                  savefile_compare="prox-v-taf-800.pdf",
                                  savefile_success="success-prox-taf-800.pdf");

  PEP.CompareZeroNoiseExperiments(files_400[1],
                                  files_400[2],
                                  loadpath = loadpath,
                                  num_n_vals = 10,
                                  dim = 400, num_tests_per = 400,
                                  plot_name_a = "Prox",
                                  plot_name_b = "TAF",
                                  savefile_compare="prox-v-taf-400.pdf",
                                  savefile_success="success-prox-taf-400.pdf");

  files_1000 =
    [ "pz_dim1000_tests100_nvals10_seed4634_proxgraph_lowtol_signs_biginit.txt",
      "pz_dim1000_tests100_nvals10_seed4634_taf_lowtol_signs_biginit.txt" ];

  files_3000 =
    [ "pz_dim3000_tests100_nvals10_seed4634_proxgraph_lowtol_signs_biginit.txt",
      "pz_dim3000_tests100_nvals10_seed4634_taf_lowtol_signs_biginit.txt" ];

  PEP.CompareZeroNoiseExperiments(files_1000[1],
                                  files_1000[2],
                                  loadpath = loadpath,
                                  num_n_vals = 10,
                                  dim = 1000, num_tests_per = 100,
                                  plot_name_a = "Prox",
                                  plot_name_b = "TAF",
                                  savefile_compare="prox-v-taf-1000.pdf",
                                  savefile_success="success-prox-taf-1000.pdf");

  PEP.CompareZeroNoiseExperiments(files_3000[1],
                                  files_3000[2],
                                  loadpath = loadpath,
                                  num_n_vals = 10,
                                  dim = 3000, num_tests_per = 100,
                                  plot_name_a = "Prox",
                                  plot_name_b = "TAF",
                                  savefile_compare="prox-v-taf-3000.pdf",
                                  savefile_success="success-prox-taf-3000.pdf");
end  # run_zero_noise

#############################################################
########### ---- Part 2: Outlier experiments ---- ###########
#############################################################

if (run_outliers)
  dims = [200, 200, 200];
  num_tests_per = [100, 100, 100];

  mtwf_files = [
    "po_dim200_tests100_pvals31_seed5155_mtw_lowtol_gauss_biginit.txt",
    "po_dim200_tests100_pvals31_seed5155_mtw_lowtol_gauss_smallinit.txt",
    "po_dim200_tests100_pvals31_seed5155_mtw_lowtol_gauss_mtwinit.txt" ];

  prox_files = [
    "po_dim200_tests100_pvals31_seed5155_proxgraph_lowtol_gauss_biginit.txt",
    "po_dim200_tests100_pvals31_seed5155_proxgraph_lowtol_gauss_smallinit.txt",
    "po_dim200_tests100_pvals31_seed5155_proxgraph_lowtol_gauss_mtwinit.txt" ];


  box_mtwf_names = [ "mtwf-big-outlier-box.pdf", "mtwf-small-outlier-box.pdf",
                     "mtwf-mtwf-outlier-box.pdf" ];
  box_prox_names = [ "prox-big-outlier-box.pdf", "prox-small-outlier-box.pdf",
                     "prox-mtwf-outlier-box.pdf" ];
  recovery_mtwf_names = [ "", "", "mtwf-mtwf-recovery.pdf" ];
  recovery_prox_names = [ "", "",  "prox-mtwf-recovery.pdf" ];

  iteration_prox_names =
    [ "prox-big-iterations.pdf", "prox-small-iterations.pdf",
      "prox-mtwf-iterations.pdf" ];

  for ii = 1:3
    close("all");
    PEP.PlotPhaseTransitionExperiment(mtwf_files[ii], loadpath=loadpath,
                                      dim = dims[ii],
                                      num_tests_per = num_tests_per[ii],
                                      save_phase_box = box_mtwf_names[ii],
                                      save_p_recovery = recovery_mtwf_names[ii],
                                      tolerance = 1e-4);
    PEP.PlotPhaseTransitionExperiment(
      prox_files[ii], loadpath=loadpath,
      dim = dims[ii],
      num_tests_per = num_tests_per[ii],
      save_phase_box = box_prox_names[ii],
      save_p_recovery = recovery_prox_names[ii],
      save_iterations = iteration_prox_names[ii],
      tolerance = 1e-4);
  end
end  # run_outliers

###########################################################
########### ---- Part 3: Digit Experiments ---- ###########
###########################################################

# Remark: make sure you have access to the values p_fail_vals, num_successes,
# num_cgs from ExplorePFailures.

if (run_digits)
  (fig, ax1) = subplots(figsize=(10, 5));
  num_experiments = max(number_digit_experiments, maximum(num_successes));
  ax1[:plot](p_fail_vals, num_successes / num_experiments, "bs-",
             linewidth=2, label="Success prob");
  ax1[:plot](p_fail_vals, ones(length(p_fail_vals)), "b-.",
             linewidth=1);
  ax1[:set_ylim](floor(minimum(num_successes) / num_experiments, 1),
                 1.05);
  ax1[:set_ylabel]("Success probability", fontsize=16);
  ax2 = ax1[:twinx]();
  ax2[:plot](p_fail_vals, num_cgs / num_experiments, "ko-",
             linewidth=2, label="Multiplies");
  ax2[:set_ylabel]("Matrix multiply count", fontsize=16);
  # fig[:tight_layout]();
  plt[:show]();
  savefig("digit-plot.pdf", bbox_inches="tight");

  close("all");

  (X, labels) = DigitExperiment.LoadDatasets();
  srand(0);
  DigitExperiment.DisplayBehaviorOfOne(X[1, :][:], k_factor = 3, p_fail = 0.0);
  figure(1);
  savefig("init-6.pdf", bbox_inches="tight");
  figure(2);
  savefig("est-6.pdf", bbox_inches="tight");
  figure(3);
  savefig("true-6.pdf", bbox_inches="tight");
  close("all");
  DigitExperiment.DisplayBehaviorOfOne(X[2, :][:], k_factor = 3, p_fail = 0.1);
  figure(1);
  savefig("init-5.pdf", bbox_inches="tight");
  figure(2);
  savefig("est-5.pdf", bbox_inches="tight");
  figure(3);
  savefig("true-5.pdf", bbox_inches="tight");
end  # run_digits

###########################################################
########### ---- Part 4: Image Experiments ---- ###########
###########################################################

if (run_image)
  filename = "./Data/rna-particles.result";

  flip_image = false;

  init_save = "rna-init.pdf";
  inacc_save = "rna-inacc.pdf";
  acc_save = "rna-acc.pdf";

  close("all");
  (rel_errs, mult_count, mult_refine_count,
   calibration_time, total_time) =
     IP.PlotResultsOfImageExperiment(filename, flip_image,
                                     init_im_save = init_save,
                                     est_im_save = inacc_save,
                                     refined_im_save = acc_save);
end
