# solvers.jl
#
# Include file for all of the basic Phase Retrieval Solver instances.

module PRS

using Convex;
using ECOS;
using Mosek;
using Base.LinAlg;
using Base.BLAS;

# First, two main files that use iterative methods to solve Phase retrieval
# objective.

# 1. Full general solvers (general A matrices)
include("robust-quadratics.jl");
# 2. Hadamard sampling operator solvers
include("hadamard-solvers.jl");
# 3. Gradient-based solvers with no noise
include("gradient-solvers.jl");
# 4. Quadratic prox-linear solvers (no noise)
include("quadratic-objs.jl");

# Helper methods for solving the intermediate QPs that arise

# Newton based solvers
include("lone-qp-solver.jl");
include("lone-qp-solver-tests.jl");

# ADMM-based solvers
include("admm-lone-solver.jl");
include("admm-lone-solver-tests.jl");

# Helper files (for matrix manipulations)
include("conjugate-solver.jl");
include("Hadamard.jl");
  
end  # module PRS
